﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace DigitalOutputExample {
	public partial class Form1 : Form {
		CommandLineOpen open;
		DigitalOutput digout = null;
		private ErrorEventBox errorBox;
        formCleaner cleaner;

        public Form1() {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
            cleaner = new formCleaner(this);
            open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {
			Phidget.InvokeEventCallbacks = true;
			errorBox = new ErrorEventBox(this);
			digout = new DigitalOutput();

			digout.Attach += output_attach;
			digout.Detach += output_detach;
			digout.Error += output_error;

			open.openCmdLine(digout);
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			digout.Close();
		}

		void output_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			DigitalOutput attachedDevice = (DigitalOutput)sender;

			outputSettings.Visible = true;

			Enum[] supportedForwardVoltages;

			switch (attachedDevice.DeviceID) { //initialize form elements based on detected device
			case DeviceID.PN_1031://led controller
			case DeviceID.PN_1032://led controller
                    supportedForwardVoltages = new Enum[] {LEDForwardVoltage.Volts_1_7,
														   LEDForwardVoltage.Volts_2_75,
														   LEDForwardVoltage.Volts_3_9,
														   LEDForwardVoltage.Volts_5_0};
				voltageCombo.DataSource = supportedForwardVoltages.ToList();
				initializeLED();
				break;
            case DeviceID.PN_LED1000://vint led controller
                supportedForwardVoltages = new Enum[] {LEDForwardVoltage.Volts_3_2,
                                                        LEDForwardVoltage.Volts_4_0,
                                                        LEDForwardVoltage.Volts_4_8,
                                                        LEDForwardVoltage.Volts_5_6};
                voltageCombo.DataSource = supportedForwardVoltages.ToList();
                initializeLED();
                break;
            }

			switch (attachedDevice.ChannelSubclass) {
			case ChannelSubclass.DigitalOutputDutyCycle:
            case ChannelSubclass.DigitalOutputLEDDriver:
				dutyCycleLbl.Visible = true;
				dutyCycleMinLbl.Visible = true;
				dutyCycleMaxLbl.Visible = true;
				dutyCycleTrk.Visible = true;
				dutyCycleTxt.Visible = true;

				dutyCycleTrk.Value = 0;
				dutyCycleTxt.Text = "0";
				break;
			default:
				dutyCycleLbl.Visible = false;
				dutyCycleMinLbl.Visible = false;
				dutyCycleMaxLbl.Visible = false;
				dutyCycleTrk.Visible = false;
				dutyCycleTxt.Visible = false;
				break;
			}

			stateBtn.Enabled = true;
		}

		void output_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
            cleaner.clean();
            foreach (Control i in this.Controls)
                i.Visible = false;
            phidgetInfoBox1.Visible = true;
		}

		void output_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		private void currentTrk_Scroll(object sender, EventArgs e) {
			try {
				digout.LEDCurrentLimit = currentTrk.Value * digout.MaxLEDCurrentLimit / (double)currentTrk.Maximum;
				currentTxt.Text = (digout.LEDCurrentLimit * 1000.0).ToString() + " mA";
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting current limit: " + ex.Message);
			}
		}

		private void dutyCycleTrk_Scroll(object sender, EventArgs e) {
			try {
				digout.DutyCycle = dutyCycleTrk.Value / (double)dutyCycleTrk.Maximum;
				dutyCycleTxt.Text = digout.DutyCycle.ToString();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting duty cycle: " + ex.Message);
			}
		}

		private void voltageCombo_SelectedIndexChanged(object sender, EventArgs e) {
			if (((ComboBox)sender).SelectedIndex == -1 || digout == null)
				return;

			try {
				digout.LEDForwardVoltage = (LEDForwardVoltage)Enum.Parse(typeof(LEDForwardVoltage), voltageCombo.SelectedValue.ToString());
			} catch (PhidgetException ex) {
				voltageCombo.SelectedIndex = -1;
				errorBox.addMessage("Error setting board forward voltage: " + ex.Message);
			}
		}

		private void initializeLED() {
			try {
				forwardVoltageLbl.Visible = true;
				voltageCombo.Visible = true;
				currentLimitLbl.Visible = true;
				currentTrk.Visible = true;
				currentTxt.Visible = true;
				currentMaxLbl.Visible = true;
				currentMaxLbl.Text = (digout.MaxLEDCurrentLimit * 1000).ToString() + " mA";
				currentMinLbl.Visible = true;
				currentMinLbl.Text = digout.MinLEDCurrentLimit.ToString() + " mA";

				currentTrk.Value = (int)(20 / (1000 * digout.MaxLEDCurrentLimit) * 1000);
				currentTxt.Text = "20 mA";
				voltageCombo.SelectedIndex = 0;
				digout.LEDCurrentLimit = 0.02;
				digout.LEDForwardVoltage = (LEDForwardVoltage)Enum.Parse(typeof(LEDForwardVoltage), voltageCombo.SelectedValue.ToString());
				digout.DutyCycle = 0;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error initializing LED: " + ex.Message);
			}
		}

		private void stateBtn_Click(object sender, EventArgs e) {
			try {
				if (stateBtn.Text == "Turn On") {
					dutyCycleTrk.Value = 1000;
					dutyCycleTxt.Text = "1";
					digout.State = !digout.State;
					stateBtn.Text = "Turn Off";
				} else {
					dutyCycleTrk.Value = 0;
					dutyCycleTxt.Text = "0";
					digout.State = !digout.State;
					stateBtn.Text = "Turn On";
				}
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting state: " + ex.Message);
			}
		}
	}
}